@@ -1,4 +1,4 @@
-package Pcore::Ext v0.1.0;
+package Pcore::Ext v0.2.0;
 
 use Pcore -dist;
 use Pcore::Ext::Context;
@@ -6,7 +6,7 @@ use JavaScript::Packer;
 
 our $SCANNED;
 
-our $NS = 'Pcore.';
+our $NS = 'Pcore';
 
 our $CFG = {
     app        => undef,
@@ -66,7 +66,10 @@ sub SCAN ( $self, $app, $framework ) {
                                 app_name => $class_name,
                             };
 
-                            $CFG->{app}->{$class_name} = { namespace => "$root_namespace\::$class_name" };
+                            $CFG->{app}->{$class_name} = {
+                                namespace => "$root_namespace\::$class_name",
+                                name      => $class_name,
+                            };
                         }
 
                         # register Ext class
@@ -111,7 +114,7 @@ sub SCAN ( $self, $app, $framework ) {
             die qq[Ext app "$namespace" requires viewport to be defined] if !$ext_map->{viewport};
 
             # store vireport class
-            $CFG->{app}->{ $namespace_cfg->{app_name} }->{viewport} = "${NS}${namespace}Viewport" =~ s/:://smgr;
+            $CFG->{app}->{ $namespace_cfg->{app_name} }->{viewport} = "${NS}.${namespace}Viewport" =~ s/:://smgr;
         }
         else {
 
@@ -132,7 +135,7 @@ sub SCAN ( $self, $app, $framework ) {
 
             my $class_name = ucfirst $class;
 
-            my $ext_class = "${NS}${namespace}${class_name}" =~ s/:://smgr;
+            my $ext_class = "${NS}.${namespace}${class_name}" =~ s/:://smgr;
 
             $CFG->{class}->{$ext_class} = {
                 class         => $ext_class,
@@ -169,7 +172,7 @@ sub SCAN ( $self, $app, $framework ) {
         # extract type from alias, alias_namespace.type
         $class->{alias_namespace} = $extend->{alias_namespace};
 
-        $class->{type} = lc "${NS}$class->{namespace}.$class->{generator}" =~ s/([.]|::)/-/smgr;
+        $class->{type} = lc "${NS}.$class->{namespace}.$class->{generator}" =~ s/([.]|::)/-/smgr;
 
         $class->{alias} = "$extend->{alias_namespace}.$class->{type}";
     }
@@ -180,18 +183,65 @@ sub SCAN ( $self, $app, $framework ) {
     for my $class ( values $CFG->{class}->%* ) {
         my $ctx = $self->_get_ctx( $class->{class}, $app, $framework );
 
-        $ctx->to_js;
+        $class->{js} = $ctx->to_js;
 
         $js_packer->minify( $ctx->{ctx}->{js}, { compress => 'obfuscate' } ) if !$app->{devel};    # clean
     }
 
     # build apps content
+    # TODO remove deps from requires upon build
     if ( !$app->{devel} ) {
-        for my $app ( values $CFG->{app}->%* ) {
+        for my $ext_app ( values $CFG->{app}->%* ) {
+            my ( %deps, %processed_class );
 
-            # TODO add requires, add app classes
+            # scan app deps
+            my $add_deps = sub ($class_name) {
+                return if exists $processed_class{$class_name};
 
-            $app->{js} = \join ';', map { $_->{js}->$* } grep { $_->{app_namespace} // q[] eq $app->{namespace} } values $CFG->{class}->%*;
+                my $class_cfg = $CFG->{class}->{$class_name};
+
+                return if !$class_cfg;
+
+                $processed_class{$class_name} = undef;
+
+                $deps{$class_name} = undef;
+
+                for my $require ( keys $class_cfg->{requires}->%* ) {
+                    next if !exists $CFG->{class}->{$require};
+
+                    $deps{$require} = undef;
+
+                    __SUB__->($require);
+                }
+
+                return;
+            };
+
+            # build app deps map
+            for my $class ( values $CFG->{class}->%* ) {
+                next if $class->{app_namespace} // $class->{namespace} ne $ext_app->{namespace};
+
+                $add_deps->( $class->{class} );
+            }
+
+            my @app_content;
+
+            # build app deps
+            for my $class ( keys %deps ) {
+                my $ctx = $self->_get_ctx( $class, $app, $framework );
+
+                my @requires = grep { !exists $deps{$_} } keys $ctx->{ctx}->{requires}->%*;
+
+                my $app_content = $ctx->to_js( \@requires );
+
+                $js_packer->minify( $app_content, { compress => 'obfuscate' } );    # clean
+
+                push @app_content, $app_content->$*;
+            }
+
+            $ext_app->{js} = \join ';', @app_content;
+
+            $ext_app->{js}->$* .= ';';
         }
     }
 
@@ -213,7 +263,7 @@ sub _get_ctx ( $self, $class, $app, $framework ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 19                   | Subroutines::ProhibitExcessComplexity - Subroutine "SCAN" with high complexity score (28)                      |
+## |    3 | 19                   | Subroutines::ProhibitExcessComplexity - Subroutine "SCAN" with high complexity score (35)                      |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
